#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "Matrix.h"
#include "LeastSquare.h"

double Poly (double a[], int n, double x)
{
	int i;
	double P;
	P = a[n];

	for (i=n-1; i>=0; i--)		
		P = P*x + a[i];

	return P;
}

void PrintPoly (double a[], int n, char* name, char* var)
{
	int i;
	
	if (name != NULL)
		printf ("%s(%s) = ", name, var);

	for (i=n; i>=0; i--) {

		if (i == n) {

			printf ("%f*%s^%i", a[i], var, i);

		} else if (i == 0) {

			if (a[i] < 0) {
				printf (" - ");
				printf ("%f\n", fabs(a[0]));			
			} else {
				printf (" + ");
				printf ("%f\n", fabs (a[0]));			
				
			}

		} else {

			if (a[i] < 0) {
				printf (" - ");
				printf ("%f*%s^%i", fabs(a[i]), var, i);			
			} else {
				printf (" + ");
				printf ("%f*%s^%i", fabs (a[i]), var, i);			
				
			}
		}
	}

	return;
}

double LeastSquare (double x[], double y[], int N, double a[], int n)
{
	int i, j, k;
	Matrix *A, *b, *u;
	double p, S, delta;

	A = MatrixNew (n+1, n+1);
	b = MatrixNew (n+1, 1);
	
	for (i=0; i<= n; i++)
		for (j=i; j<= n; j++) {			
			p = 0;
			for (k=0; k<N; k++)
				p += pow (x[k], i+j);
			MatrixSetElement (A, i, j, p); 
			if (i != j)
				MatrixSetElement (A, j, i, p); 
		}

	for (i=0; i<= n; i++) {
		p = 0;

		for (k=0; k<N; k++)
			p += pow (x[k], i)*y[k];

		MatrixSetElement (b, i, 0, p); 
	}

	u = MatrixSolve (A, b);

	if (u == NULL) {
		S = -1;
		goto end;
	}

	MatrixGetColumn (u, 0, a);

	S = 0;
	for (k = 0; k<N; k++) {
		delta = y[k] - Poly (a, n, x[k]);
		S += delta*delta;
	}

end:
	MatrixDelete (A);
	MatrixDelete (b);
	MatrixDelete (u);
	
	return S;
}

/*
  Local Variables:
  compile-command:"gcc -Wall -g -c LeastSquare.c -lm"
  End:
*/
